@@ -1,6 +1,6 @@
 use core::slice;
 
-use spirv_layout::{Module, Type, Variable};
+use spirv_layout::{LocationVariable, Module, PushConstantVariable, Type, UniformVariable};
 
 const PATH: &str = concat!(
     env!("CARGO_MANIFEST_DIR"),
@@ -13,40 +13,80 @@ fn main() {
         let words = unsafe { slice::from_raw_parts(bytes.as_ptr() as *const u32, bytes.len() / 4) };
         let module = Module::from_words(words).unwrap();
 
-        println!("=== UNIFORMS ===");
-        for var in module.get_uniforms() {
-            print_var(&module, var);
-        }
+        for ep in module.get_entry_points() {
+            println!("ENTRYPOINT {} {:?}", ep.name, ep.execution_model);
 
-        println!("=== PUSH CONSTANTS ===");
-        for var in module.get_push_constants() {
-            print_var(&module, var);
+            println!("=== INPUTS ===");
+            for var in &ep.inputs {
+                print_location_var(&module, var);
+            }
+
+            println!("=== OUTPUTS ===");
+            for var in &ep.outputs {
+                print_location_var(&module, var);
+            }
+
+            println!("=== UNIFORMS ===");
+            for var in &ep.uniforms {
+                print_uniform_var(&module, var);
+            }
+
+            println!("=== PUSH CONSTANTS ===");
+            for var in &ep.push_constants {
+                print_pc_var(&module, var);
+            }
         }
     }
 }
 
-fn print_var(module: &Module, var: &Variable) {
-    if let Some(set) = var.set {
-        if let Some(binding) = var.binding {
-            print!("layout (set={}, binding={}) ", set, binding);
-        }
-    }
+fn print_uniform_var(module: &Module, var: &UniformVariable) {
+    print!("layout (set={}, binding={}) ", var.set, var.binding);
 
     print_type(module, module.get_type(var.type_id).unwrap());
 
     println!(
-        "{};",
+        "{}; // size={}",
         if let Some(name) = &var.name {
             name
         } else {
             "<no-name>"
-        }
+        },
+        module.get_var_size(var).unwrap()
+    );
+}
+
+fn print_pc_var(module: &Module, var: &PushConstantVariable) {
+    print_type(module, module.get_type(var.type_id).unwrap());
+
+    println!(
+        "{}; // size={}",
+        if let Some(name) = &var.name {
+            name
+        } else {
+            "<no-name>"
+        },
+        module.get_var_size(var).unwrap()
+    );
+}
+
+fn print_location_var(module: &Module, var: &LocationVariable) {
+    print!("layout (location={}) ", var.location);
+
+    print_type(module, module.get_type(var.type_id).unwrap());
+
+    println!(
+        "{}; // size={}",
+        if let Some(name) = &var.name {
+            name
+        } else {
+            "<no-name>"
+        },
+        module.get_var_size(var).unwrap()
     );
 }
 
 fn print_type(module: &Module, ty: &Type) {
     match ty {
-        Type::Unknown => print!("<unknown> "),
         Type::Void => print!("void "),
         Type::Bool => print!("bool "),
         Type::Int32 => print!("int "),
@@ -74,20 +114,33 @@ fn print_type(module: &Module, ty: &Type) {
             );
 
             for elem in elements {
-                print!("    ");
+                print!("    layout(");
                 if let Some(offset) = elem.offset {
-                    print!("layout(offset={}) ", offset);
+                    print!("offset={}", offset);
                 }
+                if let Some(Type::Mat3 | Type::Mat4) = module.get_type(elem.type_id) {
+                    print!(
+                        ", {}, stride={}",
+                        if elem.row_major {
+                            "row_major"
+                        } else {
+                            "col_major"
+                        },
+                        elem.stride
+                    );
+                }
+                print!(") ");
 
                 print_type(module, module.get_type(elem.type_id).unwrap());
 
                 println!(
-                    "{};",
+                    "{}; // size={}",
                     if let Some(name) = &elem.name {
                         name
                     } else {
                         "<no-name>"
-                    }
+                    },
+                    module.get_member_size(elem).unwrap()
                 );
             }
 
@@ -100,5 +153,6 @@ fn print_type(module: &Module, ty: &Type) {
             print_type(module, module.get_type(*pointed_type_id).unwrap());
             print!("* ");
         }
+        _ => print!("<unknown> "),
     }
 }
