@@ -51,6 +51,12 @@ macro_rules! ops {
 ops!(
     5 = OpName(target: Id, name: String),
     6 = OpMemberName(target: Id, member_index: u32, name: String),
+    15 = OpEntryPoint(
+        execution_model: ExecutionModel,
+        func: Id,
+        name: String,
+        interface: Vec<Id>
+    ),
     71 = OpDecorate(target: Id, decoration: Decoration),
     72 = OpMemberDecorate(target: Id, member_index: u32, decoration: Decoration),
     19 = OpTypeVoid(result: Id),
@@ -106,7 +112,7 @@ impl DecodeArg for u32 {
     }
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone, Copy)]
 pub(crate) struct Id(pub(crate) u32);
 
 impl DecodeArg for Id {
@@ -213,6 +219,8 @@ enums!(
     Decoration {
         4 = RowMajor(),
         5 = ColMajor(),
+        7 = MatrixStride(stride: u32),
+        30 = Location(loc: u32),
         33 = Binding(binding: u32),
         34 = DescriptorSet(set: u32),
         35 = Offset(offset: u32),
@@ -230,7 +238,14 @@ enums!(
 
     StorageClass {
         0 = UniformConstant(),
+        1 = Input(),
         2 = Uniform(),
+        3 = Output(),
         9 = PushConstant(),
     },
+
+    ExecutionModel {
+        0 = Vertex(),
+        4 = Fragment(),
+    },
 );
